@@ -1,7 +1,7 @@
 # FreeSpeak - a GUI frontend to online translator engines
-# freespeak/ui/style.py - this file is part of FreeSpeak
+# freespeak/ui/style.py
 #
-## Copyright (C) 2005-2008  Luca Bruno <lethalman88@gmail.com>
+## Copyright (C) 2005, 2006, 2007, 2008, 2009  Luca Bruno <lethalman88@gmail.com>
 ##
 ## This file is part of FreeSpeak.
 ##   
@@ -17,11 +17,18 @@
 ##
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+
+"""
+Module for modifying the style of GTK+ widgets
+"""
 
 import gtk
 
 def setup_rc ():
+    """
+    Parse an RC style string
+    """
     gtk.rc_parse_string ("""
 style "tiny-button-style"
 {
@@ -30,4 +37,13 @@ style "tiny-button-style"
   ythickness = 0
 }
 widget "*.tiny-button" style "tiny-button-style"
+
+style "intro-button-style"
+{
+  GtkWidget::focus-padding = 0
+  bg[PRELIGHT] = "#fff8ae"
+  xthickness = 0
+  ythickness = 0
+}
+widget "*.intro-button" style "intro-button-style"
 """)
