@@ -1,7 +1,7 @@
 # FreeSpeak - a GUI frontend to online translator engines
-# freespeak/utils.py - this file is part of FreeSpeak
+# freespeak/utils.py
 #
-## Copyright (C) 2005-2008  Luca Bruno <lethalman88@gmail.com>
+## Copyright (C) 2005, 2006, 2007, 2008, 2009  Luca Bruno <lethalman88@gmail.com>
 ##
 ## This file is part of FreeSpeak.
 ##   
@@ -17,13 +17,29 @@
 ##
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+
+"""
+Internal utilities for FreeSpeak
+"""
 
 import gobject
 
 def syncronized (func):
+    """
+    Returns a closure to be used as decorator for methods.
+    Its purpose is to run the decorated method into the gobject mainloop
+    as an idle once a time.
+    """
     def closure (*args, **kwargs):
+        """
+        Add the 'idle' function as a gobject idle
+        """
         def idle (*aargs, **kkwargs):
+            """
+            Call the decorated function and return False to remove the
+            gobject idle.
+            """
             func (*aargs, **kkwargs)
             return False
         gobject.idle_add (idle, *args, **kwargs)
